@@ -1,103 +1,4 @@
 package Mojolicious::Plugin::Swagger2;
-
-=head1 NAME
-
-Mojolicious::Plugin::Swagger2 - Mojolicious plugin for Swagger2
-
-=head1 SYNOPSIS
-
-  package MyApp;
-  use Mojo::Base "Mojolicious";
-
-  sub startup {
-    my $app = shift;
-    $app->plugin(Swagger2 => {url => "data://MyApp/petstore.json"});
-  }
-
-  __DATA__
-  @@ petstore.json
-  {
-    "swagger": "2.0",
-    "info": {...},
-    "host": "petstore.swagger.wordnik.com",
-    "basePath": "/api",
-    "paths": {
-      "/pets": {
-        "get": {...}
-      }
-    }
-  }
-
-=head1 DESCRIPTION
-
-L<Mojolicious::Plugin::Swagger2> is L<Mojolicious::Plugin> that add routes and
-input/output validation to your L<Mojolicious> application.
-
-Have a look at the L</RESOURCES> for a complete reference to what is possible
-or look at L<Swagger2::Guides::Tutorial> for an introduction.
-
-=head1 RESOURCES
-
-=over 4
-
-=item * L<Swagger2::Guides::Tutorial> - Tutorial for this plugin
-
-=item * L<Swagger2::Guides::Render> - Details about the render process
-
-=item * L<Swagger2::Guides::ProtectedApi> - Protected API Guide
-
-=item * L<Swagger2::Guides::CustomPlaceholder> - Custom placeholder for your routes
-
-=item * L<Swagger spec|https://github.com/jhthorsen/swagger2/blob/master/t/blog/api.json>
-
-=item * L<Application|https://github.com/jhthorsen/swagger2/blob/master/t/blog/lib/Blog.pm>
-
-=item * L<Controller|https://github.com/jhthorsen/swagger2/blob/master/t/blog/lib/Blog/Controller/Posts.pm>
-
-=item * L<Tests|https://github.com/jhthorsen/swagger2/blob/master/t/authenticate.t>
-
-=back
-
-=head1 HOOKS
-
-=head2 swagger_route_added
-
-  $app->hook(swagger_route_added => sub {
-    my ($app, $r) = @_;
-    my $op_spec = $r->pattern->defaults->{swagger_operation_spec};
-    # ...
-  });
-
-The "swagger_route_added" event will be emitted on the application object
-for every route that is added by this plugin. This can be useful if you
-want to do things like specifying a custom route name.
-
-=head1 STASH VARIABLES
-
-=head2 swagger
-
-The L<Swagger2> object used to generate the routes is available
-as C<swagger> from L<stash|Mojolicious/stash>. Example code:
-
-  sub documentation {
-    my ($c, $args, $cb);
-    $c->$cb($c->stash('swagger')->pod->to_string, 200);
-  }
-
-=head2 swagger_operation_spec
-
-The Swagger specification for the current
-L<operation object|https://github.com/swagger-api/swagger-spec/blob/master/versions/2.0.md#operationObject>
-is stored in the "swagger_operation_spec" stash variable.
-
-  sub list_pets {
-    my ($c, $args, $cb);
-    $c->app->log->info($c->stash("swagger_operation_spec")->{operationId});
-    ...
-  }
-
-=cut
-
 use Mojo::Base 'Mojolicious::Plugin';
 use Mojo::JSON;
 use Mojo::Loader;
@@ -108,30 +9,9 @@ use constant DEBUG => $ENV{SWAGGER2_DEBUG} || 0;
 
 my $SKIP_OP_RE = qr(By|From|For|In|Of|To|With);
 
-=head1 ATTRIBUTES
-
-=head2 url
-
-Holds the URL to the swagger specification file.
-
-=cut
-
 has url => '';
 has _validator => sub { Swagger2::SchemaValidator->new; };
 
-=head1 HELPERS
-
-=head2 dispatch_to_swagger
-
-  $bool = $c->dispatch_to_swagger(\%data);
-
-This helper can be used to handle WebSocket requests with swagger.
-See L<Swagger2::Guides::WebSocket> for details.
-
-This helper is EXPERIMENTAL.
-
-=cut
-
 sub dispatch_to_swagger {
   return undef unless $_[1]->{op} and $_[1]->{id} and ref $_[1]->{params} eq 'HASH';
 
@@ -139,11 +19,9 @@ sub dispatch_to_swagger {
   my $self     = $c->stash('swagger.plugin');
   my $reply    = sub { $_[0]->send({json => {code => $_[2] || 200, id => $data->{id}, body => $_[1]}}) };
   my $defaults = $self->{route_defaults}{$data->{op}} or return $c->$reply(_error('Unknown operationId.'), 400);
-  my $e        = _find_action($c, $defaults->{swagger_operation_spec}, $defaults);
-  my $action   = $defaults->{action};
-  my ($input, @errors);
+  my ($e, $input, @errors);
 
-  return $c->$reply(_error($e), 501) if $e;
+  return $c->$reply(_error($e), 501) if $e = _find_action($c, $defaults);
 
   for my $p (@{$defaults->{swagger_operation_spec}{parameters} || []}) {
     my $name  = $p->{name};
@@ -156,8 +34,9 @@ sub dispatch_to_swagger {
   return $c->$reply({errors => \@errors}, 400) if @errors;
   return Mojo::IOLoop->delay(
     sub {
-      my $delay = shift;
-      my $sc = $delay->data->{sc} = $defaults->{controller}->new(%$c);
+      my $delay  = shift;
+      my $action = $defaults->{action};
+      my $sc     = $delay->data->{sc} = $defaults->{controller}->new(%$c);
       $sc->stash(swagger_operation_spec => $defaults->{swagger_operation_spec});
       $sc->$action($input, $delay->begin);
     },
@@ -174,18 +53,6 @@ sub dispatch_to_swagger {
   );
 }
 
-=head2 render_swagger
-
-  $c->render_swagger(\%err, \%data, $status);
-
-This method is used to render C<%data> from the controller method. The C<%err>
-hash will be empty on success, but can contain input/output validation errors.
-C<$status> is used to set a proper HTTP status code such as 200, 400 or 500.
-
-See also L<Swagger2::Guides::Render> for more information.
-
-=cut
-
 sub render_swagger {
   my ($c, $err, $data, $status) = @_;
 
@@ -193,73 +60,6 @@ sub render_swagger {
   return $c->render(ref $data ? (json => $data) : (text => $data), status => $status);
 }
 
-=head1 METHODS
-
-=head2 register
-
-  $self->register($app, \%config);
-
-This method is called when this plugin is registered in the L<Mojolicious>
-application.
-
-C<%config> can contain these parameters:
-
-=over 4
-
-=item * coerce
-
-This argument will be passed on to L<JSON::Validator/coerce>.
-
-=item * ensure_swagger_response
-
-  $app->plugin(swagger2 => {
-    ensure_swagger_response => {
-      exception => "Internal server error.",
-      not_found => "Not found.",
-    }
-  });
-
-Adds a L<before_render|Mojolicious/HOOKS> hook which will make sure
-"exception" and "not_found" responses are in JSON format. There is no need to
-specify "exception" and "not_found" if you are happy with the defaults.
-
-=item * route
-
-Need to hold a Mojolicious route object. See L</Protected API> for an example.
-
-This parameter is optional.
-
-=item * validate
-
-A boolean value (default is true) that will cause your schema to be validated.
-This plugin will abort loading if the schema include errors
-
-CAVEAT! There is an issue with YAML and booleans, so YAML specs might fail
-even when they are correct. See L<https://github.com/jhthorsen/swagger2/issues/39>.
-
-=item * swagger
-
-A C<Swagger2> object. This can be useful if you want to keep use the
-specification to other things in your application. Example:
-
-  use Swagger2;
-  my $swagger = Swagger2->new->load($url);
-  plugin Swagger2 => {swagger => $swagger2};
-  app->defaults(swagger_spec => $swagger->api_spec);
-
-Either this parameter or C<url> need to be present.
-
-=item * url
-
-This will be used to construct a new L<Swagger2> object. The C<url>
-can be anything that L<Swagger2/load> can handle.
-
-Either this parameter or C<swagger> need to be present.
-
-=back
-
-=cut
-
 sub register {
   my ($self, $app, $config) = @_;
   my ($base_path, $paths, $r, $swagger);
@@ -322,17 +122,11 @@ sub register {
     }
   }
 
-  # EXPERIMENTAL: Need documentation and probably a better route name()
-  if (my $title = $swagger->api_spec->get('/info/title')) {
-    my $md5
-      = Mojo::Util::md5_sum(
-      Data::Dumper->new([$swagger->api_spec->data])->Indent(0)->Pair('=>')->Purity(1)->Quotekeys(1)->Sortkeys(1)
-        ->Terse(1)->Useqq(1)->Dump);
+  if (my $spec_path = $config->{spec_path} // '/') {
+    my $title = $swagger->api_spec->get('/info/title');
     $title =~ s!\W!_!g;
-    $r->get("/$md5", [format => [qw( json )]], {format => 'json'})
-      ->to(cb => sub { shift->render(json => $swagger->api_spec->data) })->name(lc $title);
+    $r->get($spec_path)->to(cb => sub { shift->render(json => $swagger->api_spec->data) })->name(lc $title);
   }
-
   if ($config->{ensure_swagger_response}) {
     $self->_ensure_swagger_response($app, $config->{ensure_swagger_response}, $swagger);
   }
@@ -364,18 +158,17 @@ sub _generate_request_handler {
   my $defaults = {swagger_operation_spec => $op_spec};
 
   my $handler = sub {
-    my $c      = shift;
-    my $e      = _find_action($c, $op_spec, $defaults);
-    my $action = $defaults->{action};
-    my ($v, $input);
+    my $c = shift;
+    my ($e, $v, $input);
 
-    return $c->render_swagger(_error($e), {}, 501) if $e;
+    return $c->render_swagger(_error($e), {}, 501) if $e = _find_action($c, $defaults);
     $c = $defaults->{controller}->new(%$c);
     ($v, $input) = $self->_validate_input($c, $op_spec);
 
     return $c->render_swagger($v, {}, 400) if @{$v->{errors}};
     return $c->delay(
       sub {
+        my $action = $defaults->{action};
         $c->app->log->debug(qq(Swagger2 routing to controller "$defaults->{controller}" and action "$action"));
         $c->$action($input, shift->begin);
       },
@@ -525,12 +318,6 @@ sub _validate_response {
   return @errors;
 }
 
-sub _can {
-  my $defaults = shift;
-  return if $defaults->{controller}->can($defaults->{action});
-  return qq(Method "$defaults->{action}" not implemented.);
-}
-
 sub _coerce_by_collection_format {
   my ($value, $schema) = @_;
   my $re = $Swagger2::SchemaValidator::COLLECTION_RE{$schema->{collectionFormat}} || '';
@@ -543,36 +330,34 @@ sub _coerce_by_collection_format {
   return \@data;
 }
 
-sub _die {
-  die "$_[1]: ", Mojo::Util::dumper($_[0]);
-}
-
 sub _error {
   return {errors => [{message => $_[0], path => '/'}]};
 }
 
 sub _find_action {
-  return if $_[2]->{controller};    # cached
-
-  my ($c, $op_spec, $defaults) = @_;
-  my $op = $op_spec->{operationId} or return 'operationId is missing.';
+  return if $_[1]->{controller};    # cached
+  my ($c, $defaults) = @_;
+  my $op = $defaults->{swagger_operation_spec}{operationId} or return 'operationId is missing.';
+  my $can = sub {
+    $defaults->{controller}->can($defaults->{action}) ? '' : qq(Method "$defaults->{action}" not implemented.);
+  };
 
   # specify controller manually
-  @$defaults{qw( action controller )} = _load($c, $op, $op_spec->{'x-mojo-controller'});
-  return _can($defaults) if $defaults->{controller};
+  @$defaults{qw( action controller )} = _load($c, $op, $defaults->{swagger_operation_spec}{'x-mojo-controller'});
+  return $can->() if $defaults->{controller};
 
   # "createFileInFileSystem" = ("createFile", "FileSystem")
   @$defaults{qw( action controller )} = _load($c, split $SKIP_OP_RE, $op);
-  return _can($defaults) if $defaults->{controller};
+  return $can->() if $defaults->{controller};
 
   # "showPetById" = "showPet"
   $op =~ s!$SKIP_OP_RE.*$!!;
 
   # "show_fooPet" = ("show_foo", "Pet")
   @$defaults{qw( action controller )} = _load($c, $op =~ /^([a-z_]+)([A-Z]\w+)$/);
-  return _can($defaults) if $defaults->{controller};
+  return $can->() if $defaults->{controller};
 
-  return qq(Controller from operationId "$op_spec->{operationId}" could not be loaded.);
+  return qq(Controller from operationId "$defaults->{swagger_operation_spec}{operationId}" could not be loaded.);
 }
 
 sub _load {
@@ -601,6 +386,204 @@ sub _load {
   return;
 }
 
+1;
+
+=encoding utf8
+
+=head1 NAME
+
+Mojolicious::Plugin::Swagger2 - Mojolicious plugin for Swagger2
+
+=head1 SYNOPSIS
+
+  package MyApp;
+  use Mojo::Base "Mojolicious";
+
+  sub startup {
+    my $app = shift;
+    $app->plugin(Swagger2 => {url => "data://MyApp/petstore.json"});
+  }
+
+  __DATA__
+  @@ petstore.json
+  {
+    "swagger": "2.0",
+    "info": {...},
+    "host": "petstore.swagger.wordnik.com",
+    "basePath": "/api",
+    "paths": {
+      "/pets": {
+        "get": {...}
+      }
+    }
+  }
+
+=head1 DESCRIPTION
+
+L<Mojolicious::Plugin::Swagger2> is L<Mojolicious::Plugin> that add routes and
+input/output validation to your L<Mojolicious> application.
+
+Have a look at the L</RESOURCES> for a complete reference to what is possible
+or look at L<Swagger2::Guides::Tutorial> for an introduction.
+
+=head1 RESOURCES
+
+=over 4
+
+=item * L<Swagger2::Guides::Tutorial> - Tutorial for this plugin
+
+=item * L<Swagger2::Guides::Render> - Details about the render process
+
+=item * L<Swagger2::Guides::ProtectedApi> - Protected API Guide
+
+=item * L<Swagger2::Guides::CustomPlaceholder> - Custom placeholder for your routes
+
+=item * L<Swagger spec|https://github.com/jhthorsen/swagger2/blob/master/t/blog/api.json>
+
+=item * L<Application|https://github.com/jhthorsen/swagger2/blob/master/t/blog/lib/Blog.pm>
+
+=item * L<Controller|https://github.com/jhthorsen/swagger2/blob/master/t/blog/lib/Blog/Controller/Posts.pm>
+
+=item * L<Tests|https://github.com/jhthorsen/swagger2/blob/master/t/authenticate.t>
+
+=back
+
+=head1 HOOKS
+
+=head2 swagger_route_added
+
+  $app->hook(swagger_route_added => sub {
+    my ($app, $r) = @_;
+    my $op_spec = $r->pattern->defaults->{swagger_operation_spec};
+    # ...
+  });
+
+The "swagger_route_added" event will be emitted on the application object
+for every route that is added by this plugin. This can be useful if you
+want to do things like specifying a custom route name.
+
+=head1 STASH VARIABLES
+
+=head2 swagger
+
+The L<Swagger2> object used to generate the routes is available
+as C<swagger> from L<stash|Mojolicious/stash>. Example code:
+
+  sub documentation {
+    my ($c, $args, $cb);
+    $c->$cb($c->stash('swagger')->pod->to_string, 200);
+  }
+
+=head2 swagger_operation_spec
+
+The Swagger specification for the current
+L<operation object|https://github.com/swagger-api/swagger-spec/blob/master/versions/2.0.md#operationObject>
+is stored in the "swagger_operation_spec" stash variable.
+
+  sub list_pets {
+    my ($c, $args, $cb);
+    $c->app->log->info($c->stash("swagger_operation_spec")->{operationId});
+    ...
+  }
+
+=head1 ATTRIBUTES
+
+=head2 url
+
+Holds the URL to the swagger specification file.
+
+=head1 HELPERS
+
+=head2 dispatch_to_swagger
+
+  $bool = $c->dispatch_to_swagger(\%data);
+
+This helper can be used to handle WebSocket requests with swagger.
+See L<Swagger2::Guides::WebSocket> for details.
+
+This helper is EXPERIMENTAL.
+
+=head2 render_swagger
+
+  $c->render_swagger(\%err, \%data, $status);
+
+This method is used to render C<%data> from the controller method. The C<%err>
+hash will be empty on success, but can contain input/output validation errors.
+C<$status> is used to set a proper HTTP status code such as 200, 400 or 500.
+
+See also L<Swagger2::Guides::Render> for more information.
+
+=head1 METHODS
+
+=head2 register
+
+  $self->register($app, \%config);
+
+This method is called when this plugin is registered in the L<Mojolicious>
+application.
+
+C<%config> can contain these parameters:
+
+=over 4
+
+=item * coerce
+
+This argument will be passed on to L<JSON::Validator/coerce>.
+
+=item * ensure_swagger_response
+
+  $app->plugin(swagger2 => {
+    ensure_swagger_response => {
+      exception => "Internal server error.",
+      not_found => "Not found.",
+    }
+  });
+
+Adds a L<before_render|Mojolicious/HOOKS> hook which will make sure
+"exception" and "not_found" responses are in JSON format. There is no need to
+specify "exception" and "not_found" if you are happy with the defaults.
+
+=item * route
+
+Need to hold a Mojolicious route object. See L</Protected API> for an example.
+
+This parameter is optional.
+
+=item * spec_path
+
+Holds the location for where the specifiation can be served from. The default
+value is "/", relative to "basePath" in the specification. Can be disabled
+by setting this value to empty string.
+
+=item * validate
+
+A boolean value (default is true) that will cause your schema to be validated.
+This plugin will abort loading if the schema include errors
+
+CAVEAT! There is an issue with YAML and booleans, so YAML specs might fail
+even when they are correct. See L<https://github.com/jhthorsen/swagger2/issues/39>.
+
+=item * swagger
+
+A C<Swagger2> object. This can be useful if you want to keep use the
+specification to other things in your application. Example:
+
+  use Swagger2;
+  my $swagger = Swagger2->new->load($url);
+  plugin Swagger2 => {swagger => $swagger2};
+  app->defaults(swagger_spec => $swagger->api_spec);
+
+Either this parameter or C<url> need to be present.
+
+=item * url
+
+This will be used to construct a new L<Swagger2> object. The C<url>
+can be anything that L<Swagger2/load> can handle.
+
+Either this parameter or C<swagger> need to be present.
+
+=back
+
 =head1 COPYRIGHT AND LICENSE
 
 Copyright (C) 2014-2015, Jan Henning Thorsen
@@ -613,5 +596,3 @@ the terms of the Artistic License version 2.0.
 Jan Henning Thorsen - C<jhthorsen@cpan.org>
 
 =cut
-
-1;
