@@ -1,5 +1,6 @@
 package Swagger2::Client;
 use Mojo::Base -base;
+use Mojo::JSON;
 use Mojo::UserAgent;
 use Mojo::Util;
 use Carp ();
@@ -22,11 +23,8 @@ sub generate {
   $generated =~ s!\W!_!g;
   $generated = "$class\::$generated";
 
-  if ($generated->isa($class)) {
-    return $generated->new;
-  }
-
-  eval "package $generated; use Mojo::Base '$class'; 1" or die "package $generated: $@";
+  return $generated->new if $generated->isa($class);                   # already generated
+  _init_package($generated, $class);
   Mojo::Util::monkey_patch($generated, _swagger => sub {$swagger});
 
   for my $path (keys %$paths) {
@@ -60,7 +58,10 @@ sub _generate_method {
     my @e    = $self->_validate_request($args, $op_spec, $req);
 
     if (@e) {
-      Carp::croak('Invalid input: ' . join ' ', @e) unless $cb;
+      unless ($cb) {
+        return _invalid_input_res(\@e) if $self->return_on_error;
+        Carp::croak('Invalid input: ' . join ' ', @e);
+      }
       $self->$cb(\@e, undef);
       return $self;
     }
@@ -81,12 +82,29 @@ sub _generate_method {
     }
     else {
       my $tx = $self->ua->$http_method(@$req);
-      Carp::croak(join ': ', grep {defined} $tx->error->{message}, $tx->res->body) if $tx->error;
-      return $tx->res;
+      return $tx->res if !$tx->error or $self->return_on_error;
+      Carp::croak(join ': ', grep {defined} $tx->error->{message}, $tx->res->body);
     }
   };
 }
 
+sub _init_package {
+  my ($package, $base) = @_;
+  eval <<"HERE" or die "package $package: $@";
+package $package;
+use Mojo::Base '$base';
+has return_on_error => 0;
+1;
+HERE
+}
+
+sub _invalid_input_res {
+  my $res = Mojo::Message::Response->new;
+  $res->body(Mojo::JSON::encode_json({errors => $_[0]}));
+  $res->code(400)->message($res->default_message);
+  $res->error({message => 'Invalid input', code => 400});
+}
+
 sub _swagger_url {
   if (UNIVERSAL::isa($_[0], 'Swagger2')) {
     my $swagger = shift->load->expand;
@@ -196,20 +214,28 @@ The swagger specification will the be turned into a sub class of
 L<Swagger2::Client>, where the "parameters" rules are used to do input
 validation.
 
-The method name added is a L<decamelized|Mojo::Util/decamelize> version of
-the operationId, which creates a more perl-ish feeling to the API.
-
   use Swagger2::Client;
-  my $ua = Swagger2::Client->generate("file:///path/to/api.json");
+  $ua = Swagger2::Client->generate("file:///path/to/api.json");
 
-  # blocking
-  my $pets = $ua->list_pets; # instead of listPets()
+  # blocking (will croak() on error)
+  $pets = $ua->listPets;
+
+  # blocking (will not croak() on error)
+  $ua->return_on_error(1);
+  $pets = $ua->listPets;
 
   # non-blocking
-  $ua = $ua->list_pets(sub { my ($ua, $err, $pets) = @_; });
+  $ua = $ua->listPets(sub { my ($ua, $err, $pets) = @_; });
 
   # with arguments, where the key map to the "parameters" name
-  my $pets = $ua->list_pets({limit => 10});
+  $pets = $ua->listPets({limit => 10});
+
+The method name added will both be the original C<operationId>, but a "snake
+case" version will also be added. Example:
+
+  "operationId": "listPets"
+    => $client->listPets()
+    => $client->list_pets()
 
 =head2 Customization
 
